@@ -1,14 +1,14 @@
 import asyncio
 import re
 from pathlib import Path
-from typing import Any, Dict, List
+from typing import Any
 
 from ops2deb import logger
 from ops2deb.exceptions import Ops2debBuilderError
 from ops2deb.utils import log_and_raise
 
 
-def parse_debian_control(cwd: Path) -> Dict[str, str]:
+def parse_debian_control(cwd: Path) -> dict[str, str]:
     """
     Extract fields from debian/control
     :param cwd: Path to debian source package
@@ -56,7 +56,7 @@ async def build_package(cwd: Path) -> None:
         logger.info(f"Successfully built {str(cwd)}")
 
 
-def build(package_paths: List[Path], workers: int) -> None:
+def build(package_paths: list[Path], workers: int) -> None:
     """
     Build debian source packages in parallel.
     :param package_paths: list of debian source package paths
@@ -87,9 +87,11 @@ def build_all(output_directory: Path, workers: int) -> None:
     :param output_directory: path where to search for source packages
     :param workers: Number of threads to run in parallel
     """
+    if output_directory.exists() is False:
+        raise Ops2debBuilderError(f"Directory {output_directory} does not exist")
 
     if output_directory.is_dir() is False:
-        raise Ops2debBuilderError(f"Directory {output_directory} does not exist")
+        raise Ops2debBuilderError(f"{output_directory} is not a directory")
 
     paths = []
     for output_directory in output_directory.iterdir():
