@@ -1,24 +1,13 @@
 import json
 from pathlib import Path
 from textwrap import wrap
-from typing import Any, Tuple
+from typing import Any, Callable, Optional, Tuple
 
 import yaml
 
 from ops2deb.exceptions import Ops2debFormatterError
 from ops2deb.parser import Blueprint, load, validate
-
-
-class PrettyYAMLDumper(yaml.dumper.SafeDumper):
-    def expect_block_sequence(self) -> None:
-        self.increase_indent(flow=False, indentless=False)
-        self.state = self.expect_first_block_sequence_item
-
-    def choose_scalar_style(self) -> str:
-        style: str = super().choose_scalar_style()
-        style = '"' if style == "'" else style
-        style = "|" if self.analysis.multiline else style
-        return style
+from ops2deb.utils import PrettyYAMLDumper
 
 
 def sort_blueprints(blueprints: list[dict[str, Any]]) -> list[dict[str, Any]]:
@@ -37,9 +26,13 @@ def format_description(description: str) -> str:
 
 
 def format_blueprint(blueprint: dict[str, Any]) -> dict[str, Any]:
+    if blueprint_arch := blueprint.pop("arch", None):
+        blueprint["architecture"] = blueprint_arch
     blueprint = json.loads(Blueprint.construct(**blueprint).json(exclude_defaults=True))
-    if "description" in blueprint.keys():
-        blueprint["description"] = format_description(blueprint["description"])
+    if (blueprint_fetch := blueprint.get("fetch", None)) and len(blueprint_fetch) == 1:
+        blueprint["fetch"] = blueprint_fetch["url"]
+    if blueprint_desc := blueprint.get("description", None):
+        blueprint["description"] = format_description(blueprint_desc)
     keys_to_remove: list[str] = []
     for key, value in blueprint.items():
         if isinstance(value, list) and not blueprint.get(key):
@@ -49,7 +42,10 @@ def format_blueprint(blueprint: dict[str, Any]) -> dict[str, Any]:
     return blueprint
 
 
-def format(configuration_path: Path) -> None:
+def format(
+    configuration_path: Path,
+    additional_blueprint_formatting: Optional[Callable[[dict[str, Any]], None]] = None,
+) -> None:
     configuration_dict = load(configuration_path)
     validate(configuration_dict)
 
@@ -63,6 +59,10 @@ def format(configuration_path: Path) -> None:
     # sort blueprints by name, version and revision
     raw_blueprints = sort_blueprints(raw_blueprints)
 
+    if additional_blueprint_formatting is not None:
+        for blueprint in raw_blueprints:
+            additional_blueprint_formatting(blueprint)
+
     # wrap descriptions, remove default values, remove empty lists
     raw_blueprints = [format_blueprint(b) for b in raw_blueprints]
 
