@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Any, Literal
+from typing import Any, Literal, Sequence
 
 from pydantic import AnyHttpUrl, BaseModel, Field, ValidationError, validator
 from ruamel.yaml import YAML, YAMLError  # type: ignore[attr-defined]
@@ -13,7 +13,7 @@ Architecture = Literal["all", "amd64", "arm64", "armhf"]
 class Base(BaseModel):
     class Config:
         extra = "forbid"
-        allow_mutation = False
+        frozen = True
         anystr_strip_whitespace = True
 
 
@@ -25,12 +25,14 @@ class ArchitectureMap(Base):
 
 class RemoteFile(Base):
     url: AnyHttpUrl = Field(..., description="URL template of the file")
-    sha256: str = Field(..., description="SHA256 checksum of the file")
+    sha256: str = Field(..., description="SHA256 checksum of the file (deprecated)")
 
 
-class MultiArchitectureRemoteFile(Base):
+class MultiArchitectureFetch(Base):
     url: AnyHttpUrl = Field(..., description="URL template of the file")
-    sha256: ArchitectureMap = Field(..., description="SHA256 checksum of each file")
+    sha256: ArchitectureMap | None = Field(
+        None, description="SHA256 checksum of each file (deprecated)"
+    )
     targets: ArchitectureMap | None = Field(
         None,
         description="Architecture to target name map. The URL can be templated with "
@@ -72,13 +74,23 @@ class HereDocument(Base):
         return self.path
 
 
+class Matrix(Base):
+    architectures: list[Architecture] = Field(
+        default_factory=list, description="List of architectures"
+    )
+
+
 class Blueprint(Base):
     name: str = Field(..., description="Package name")
+    matrix: Matrix | None = Field(
+        None, description="Generate multiple packages from one a single blueprint"
+    )
     version: str = Field(..., description="Package name")
-    homepage: AnyHttpUrl | None = Field(None, description="Upstream project homepage")
     revision: int = Field(1, description="Package revision", ge=1)
     epoch: int = Field(0, description="Package epoch", ge=0)
-    arch: Architecture = Field("amd64", description="Package architecture")
+    architecture: Architecture = Field("amd64", description="Package architecture")
+    arch: Architecture = Field("amd64", description="Package architecture (deprecated)")
+    homepage: AnyHttpUrl | None = Field(None, description="Upstream project homepage")
     summary: str = Field(..., description="Package short description, one line only")
     description: str = Field("", description="Package description")
     build_depends: list[str] = Field(
@@ -101,15 +113,22 @@ class Blueprint(Base):
         description="Conflicting packages, for more information read "
         "https://www.debian.org/doc/debian-policy/ch-relationships.html",
     )
-    fetch: RemoteFile | MultiArchitectureRemoteFile | None = Field(
+    fetch: AnyHttpUrl | MultiArchitectureFetch | RemoteFile | None = Field(
         None,
-        description="Describe a file (or a file per architecture) to download before "
-        "running the build script",
+        description="Url of a file (or a file per architecture) to download before "
+        "running build and install instructions",
     )
     install: list[HereDocument | SourceDestinationStr] = Field(default_factory=list)
     script: list[str] = Field(default_factory=list, description="Build instructions")
 
-    @validator("*", pre=True)
+    @validator("arch", "architecture", pre=False, always=False)
+    def _archs_cannot_be_used_with_arch(cls, v: Any, values: Any) -> Any:
+        if values["matrix"] and values["matrix"]["architectures"]:
+            raise ValueError("'architectures' cannot be used with 'architecture'")
+        values["architecture"] = v
+        return v
+
+    @validator("name", "version", "summary", "description", "homepage", pre=True)
     def _render_string_attributes(cls, v: Any) -> Any:
         if isinstance(v, str):
             return environment.from_string(v).render()
@@ -117,47 +136,55 @@ class Blueprint(Base):
 
     def _get_additional_variables(self) -> dict[str, str | None]:
         target = (
-            (getattr(self.fetch.targets, self.arch, self.arch) or self.arch)
-            if isinstance(self.fetch, MultiArchitectureRemoteFile)
-            else self.arch
+            (
+                getattr(self.fetch.targets, self.architecture, self.architecture)
+                or self.architecture
+            )
+            if isinstance(self.fetch, MultiArchitectureFetch)
+            else self.architecture
         )
         return dict(
             target=target,
-            goarch=DEFAULT_GOARCH_MAP.get(self.arch, None),
-            rust_target=DEFAULT_RUST_TARGET_MAP.get(self.arch, None),
+            goarch=DEFAULT_GOARCH_MAP.get(self.architecture, None),
+            rust_target=DEFAULT_RUST_TARGET_MAP.get(self.architecture, None),
         )
 
-    def supported_architectures(self) -> list[str]:
-        if isinstance(self.fetch, MultiArchitectureRemoteFile):
-            return list(self.fetch.sha256.dict(exclude_none=True).keys())
-        else:
-            return [self.arch]
+    def architectures(self) -> Sequence[str]:
+        if self.matrix and self.matrix.architectures:
+            return self.matrix.architectures
+        elif isinstance(self.fetch, MultiArchitectureFetch):
+            if self.fetch.sha256 is not None:
+                return list(self.fetch.sha256.dict(exclude_none=True).keys())
+        return [self.architecture]
 
     def render_string(self, string: str, **kwargs: str | Path | None) -> str:
         version = kwargs.pop("version", None)
         version = version or self.version
         return environment.from_string(string).render(
             name=self.name,
-            arch=self.arch,
+            arch=self.architecture,
             version=version,
             **(self._get_additional_variables() | kwargs),
         )
 
-    def render_fetch(self, version: str | None = None) -> RemoteFile | None:
+    def render_fetch_url(self, version: str | None = None) -> str | None:
         if self.fetch is None:
             return None
-        if isinstance(self.fetch, MultiArchitectureRemoteFile):
-            sha256 = getattr(self.fetch.sha256, self.arch)
-        else:
+        url = self.fetch if isinstance(self.fetch, str) else self.fetch.url
+        return self.render_string(url, version=version)
+
+    def render_fetch(self, version: str | None = None) -> str | RemoteFile | None:
+        if self.fetch is None:
+            return None
+        if isinstance(self.fetch, MultiArchitectureFetch):
+            sha256 = getattr(self.fetch.sha256, self.architecture)
+        elif isinstance(self.fetch, RemoteFile):
             sha256 = self.fetch.sha256
+        else:
+            return self.render_fetch_url(version=version)
         if sha256 is None:
             return None
-        url = self.render_string(
-            self.fetch.url,
-            version=version,
-            sha256=sha256,
-        )
-        return RemoteFile(url=url, sha256=sha256)
+        return RemoteFile(url=self.render_fetch_url(version), sha256=sha256)
 
 
 class Configuration(Base):
@@ -167,8 +194,8 @@ class Configuration(Base):
 def extend(blueprints: list[Blueprint]) -> list[Blueprint]:
     extended_list: list[Blueprint] = []
     for blueprint in blueprints:
-        for arch in blueprint.supported_architectures():
-            extended_list.append(blueprint.copy(update={"arch": arch}))
+        for arch in blueprint.architectures():
+            extended_list.append(blueprint.copy(update={"architecture": arch}))
     return extended_list
 
 
