@@ -15,8 +15,9 @@ from semver.version import Version
 from ops2deb import logger
 from ops2deb.client import client_factory
 from ops2deb.exceptions import Ops2debError, Ops2debUpdaterError
-from ops2deb.fetcher import FetchResult, fetch_urls
-from ops2deb.parser import Blueprint, RemoteFile, load, validate
+from ops2deb.fetcher import Fetcher, FetchResult
+from ops2deb.lockfile import Lock
+from ops2deb.parser import Blueprint, load, validate
 from ops2deb.utils import separate_results_from_errors
 
 
@@ -32,9 +33,8 @@ class BaseUpdateStrategy:
         self.client = client
 
     async def _try_version(self, blueprint: Blueprint, version: str) -> bool:
-        if not (remote_file := blueprint.render_fetch(version=version)):
+        if not (url := blueprint.render_fetch_url(version=version)):
             return False
-        url = remote_file.url
         logger.debug(f"{self.__class__.__name__} - {blueprint.name} - Trying {url}")
         try:
             response = await self.client.head(url)
@@ -115,10 +115,10 @@ class GithubUpdateStrategy(BaseUpdateStrategy):
 
     @classmethod
     def _get_github_repo_api_base_url(cls, blueprint: Blueprint) -> str:
-        if (fetch := blueprint.render_fetch()) is None:
+        if (url := blueprint.render_fetch_url()) is None:
             raise ValueError(f"Blueprint {blueprint.name} has no fetch instruction")
-        if (match := re.match(cls.github_url_re, fetch.url)) is None:
-            raise ValueError(f"URL {fetch.url} is not supported")
+        if (match := re.match(cls.github_url_re, url)) is None:
+            raise ValueError(f"URL {url} is not supported")
         return f"{cls.github_base_api_url}/repos/{match['owner']}/{match['name']}"
 
     async def _get_latest_github_release(self, blueprint: Blueprint) -> dict[str, str]:
@@ -166,7 +166,7 @@ class LatestRelease:
     blueprint_index: int
     blueprint: Blueprint
     version: str
-    fetch_results: dict[str, FetchResult]
+    fetch_results: list[FetchResult]
 
     def update_configuration(
         self, blueprint_dict: dict[str, Any] | list[dict[str, Any]]
@@ -178,17 +178,19 @@ class LatestRelease:
             else blueprint_dict
         )
 
-        new_sha256_object: Any = {}
-        for arch, fetch_result in self.fetch_results.items():
-            if isinstance(self.blueprint.fetch, RemoteFile):
-                new_sha256_object = fetch_result.sha256_sum
-            else:
-                new_sha256_object[arch] = fetch_result.sha256_sum
-
-        raw_blueprint["fetch"]["sha256"] = new_sha256_object
         raw_blueprint["version"] = self.version
         raw_blueprint.pop("revision", None)
 
+        # TODO: remove this when fetch.sha256 support is dropped
+        if not isinstance(raw_blueprint["fetch"], str):
+            raw_blueprint["fetch"].pop("sha256", None)
+            if len(architectures := self.blueprint.architectures()) > 1:
+                raw_blueprint["matrix"] = {"architectures": architectures}
+                raw_blueprint.pop("architecture", None)
+                raw_blueprint.pop("arch", None)
+            if len(raw_blueprint["fetch"]) == 1:
+                raw_blueprint["fetch"] = raw_blueprint["fetch"]["url"]
+
 
 async def _find_latest_version(client: httpx.AsyncClient, blueprint: Blueprint) -> str:
     strategies = [GithubUpdateStrategy(client), GenericUpdateStrategy(client)]
@@ -214,8 +216,16 @@ async def _find_latest_version(client: httpx.AsyncClient, blueprint: Blueprint)
     raise Ops2debUpdaterError(error)
 
 
+def _blueprint_fetch_urls(blueprint: Blueprint, version: str | None = None) -> list[str]:
+    urls: list[str] = []
+    for architecture in blueprint.architectures():
+        blueprint = blueprint.copy(update={"architecture": architecture})
+        urls.append(str(blueprint.render_fetch_url(version)))
+    return urls
+
+
 async def _find_latest_releases(
-    blueprint_list: list[Blueprint], skip_names: list[str] | None = None
+    blueprint_list: list[Blueprint], fetcher: Fetcher, skip_names: list[str] | None = None
 ) -> Tuple[list[LatestRelease], dict[int, Ops2debError]]:
     skip_names = skip_names or []
     blueprints = {
@@ -235,20 +245,17 @@ async def _find_latest_releases(
     blueprints = {i: b for i, b in blueprints.items() if versions[i] != b.version}
 
     # gather the urls of files we need to download to get the new checksums
-    urls: dict[int, dict[str, str]] = defaultdict(dict)
+    urls: dict[int, list[str]] = defaultdict(list)
     for index, blueprint in blueprints.items():
-        for arch in blueprint.supported_architectures():
-            blueprint = blueprint.copy(update={"arch": arch})
-            remote_file = cast(RemoteFile, blueprint.render_fetch(versions[index]))
-            urls[index][arch] = str(remote_file.url)
+        urls[index] = _blueprint_fetch_urls(blueprint, versions[index])
 
-    url_list = list(itertools.chain(*[u.values() for u in urls.values()]))
-    results, fetch_errors = await fetch_urls(url_list)
+    url_list = list(itertools.chain(*[u for u in urls.values()]))
+    results, fetch_errors = await fetcher.fetch_urls(url_list)
 
     # remove blueprint we can't update because we could not fetch associated files
     for failed_url, exception in fetch_errors.items():
         for index, blueprint_urls in urls.items():
-            if failed_url in blueprint_urls.values():
+            if failed_url in blueprint_urls:
                 errors[index] = exception
     blueprints = {i: b for i, b in blueprints.items() if i not in errors.keys()}
 
@@ -259,7 +266,7 @@ async def _find_latest_releases(
                 blueprint_index=index,
                 blueprint=blueprint,
                 version=versions[index],
-                fetch_results={arch: results[url] for arch, url in urls[index].items()},
+                fetch_results=[results[url] for url in urls[index]],
             )
         )
 
@@ -267,13 +274,15 @@ async def _find_latest_releases(
 
 
 def find_latest_releases(
-    blueprint_list: list[Blueprint], skip_names: list[str] | None = None
+    blueprint_list: list[Blueprint], fetcher: Fetcher, skip_names: list[str] | None = None
 ) -> Tuple[list[LatestRelease], dict[int, Ops2debError]]:
-    return asyncio.run(_find_latest_releases(blueprint_list, skip_names))
+    return asyncio.run(_find_latest_releases(blueprint_list, fetcher, skip_names))
 
 
 def update(
     configuration_path: Path,
+    lockfile_path: Path,
+    fetcher: Fetcher,
     dry_run: bool = False,
     output_path: Path | None = None,
     skip_names: list[str] | None = None,
@@ -285,17 +294,21 @@ def update(
     blueprints = validate(configuration_dict)
 
     logger.title("Looking for new releases...")
-    releases, errors = find_latest_releases(blueprints, skip_names)
+    releases, errors = find_latest_releases(blueprints, fetcher, skip_names)
     if not releases:
         logger.info("Did not found any updates")
 
     if dry_run is False and releases:
+        lock = Lock(lockfile_path)
         for release in releases:
             release.update_configuration(configuration_dict)
+            lock.add(release.fetch_results)
+            lock.remove(_blueprint_fetch_urls(release.blueprint))
         with configuration_path.open("w") as output:
             yaml.dump(configuration_dict, output)
+        lock.save()
 
-        logger.info("Configuration file updated")
+        logger.info("Lockfile and configuration updated")
 
     if output_path is not None:
         lines = [
