@@ -3,6 +3,8 @@ from contextlib import contextmanager
 from pathlib import Path
 from typing import Iterator, Tuple, TypeVar
 
+import yaml
+
 from ops2deb import logger
 from ops2deb.exceptions import Ops2debError
 
@@ -39,3 +41,15 @@ def working_directory(path: Path) -> Iterator[None]:
         yield
     finally:
         os.chdir(origin)
+
+
+class PrettyYAMLDumper(yaml.dumper.SafeDumper):
+    def expect_block_sequence(self) -> None:
+        self.increase_indent(flow=False, indentless=False)
+        self.state = self.expect_first_block_sequence_item
+
+    def choose_scalar_style(self) -> str:
+        style: str = super().choose_scalar_style()
+        style = '"' if style == "'" else style
+        style = "|" if self.analysis.multiline else style
+        return style
